package org.apidesign.apifest08.test;

import junit.framework.TestCase;
import org.apidesign.apifest08.currency.Amount;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.ConvertorFactory;

/** Finish the Convertor API, and then write bodies of methods inside
 * of this class to match the given tasks. To fullfil your task, use the
 * API define in the <code>org.apidesign.apifest08.currency</code> package.
 * Do not you reflection, or other hacks as your code
 * shall run without any runtime permissions.
 */
public class Task1Test extends TestCase {
    public Task1Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    /** Create convertor that understands two currencies, CZK and
     *  USD. Make 1 USD == 17 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting USD to CZK and CZK to USD
     */
    public static Convertor createCZKtoUSD() {
        return ConvertorFactory.createConvertor(17);
    }

    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 80 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     * 
     * @return prepared convertor ready for converting SKK to CZK and CZK to SKK
     */
    public static Convertor createSKKtoCZK() {
        return ConvertorFactory.createConvertor(0.8);
    }
    
    /** Use the convertor from <code>createCZKtoUSD</code> method and do few conversions
     * with it.
     */
    public void testCurrencyCZKUSD() throws Exception {
        Convertor c = createCZKtoUSD();
        // convert $5 to CZK using c:
        assertEquals("Result is 85 CZK", 85l, c.convert(new Amount(5)).getAmount());

        // convert $8 to CZK
        assertEquals("Result is 136 CZK", 136l, c.convert(new Amount(8)).getAmount());

        // convert 1003CZK to USD
        assertEquals("Result is 136 CZK", 59l, c.convertBack(new Amount(1003)).getAmount());
        // assertEquals("Result is 59 USD");
    }

    /** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZK() throws Exception {
        Convertor c = createSKKtoCZK();
        // convert 16CZK using c:
        assertEquals("Result is 20 SKK", 20l, c.convertBack(new Amount(16)).getAmount());

        // convert 500SKK to CZK
        assertEquals("Result is 400 CZK", 400l, c.convert(new Amount(500)).getAmount());
    }

    /** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZKwithPences() throws Exception {
        Convertor c = createSKKtoCZK();
        // convert 16CZK 16h using c:
        assertEquals("Result is 20 SKK 20h", new Amount(20, 20), c.convertBack(new Amount(16, 16)));

        // convert 500SKK 80h to CZK
        assertEquals("Result is 400 CZK 64h", new Amount(400, 64), c.convert(new Amount(500, 80)));

        // convert 400CZK 80h to SKK
        assertEquals("Result is 501 CZK", new Amount(501), c.convertBack(new Amount(400, 80)));
    }

    public void testNegativeSKKCZ() throws Exception{
        Convertor c = createSKKtoCZK();
        // convert -16CZK using c:
        assertEquals("Result is -20", new Amount(-20), c.convertBack(new Amount(-16)));

        // convert -500SKK 80h to CZK
        assertEquals("Result is -400CZK 64h", new Amount(-400, 64), c.convert(new Amount(-500, 80)));

    }
}

